﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Syncfusion.Windows.Tools.Controls;
using Syncfusion.Windows.Shared;
using System.Diagnostics;

namespace RichTextBoxAdvSample
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : RibbonWindow
    {
        Syncfusion.Windows.Tools.Controls.RoutedCommand m_VisualStyle;
        public Syncfusion.Windows.Tools.Controls.RoutedCommand VisualStyleCommand
        {
            get
            {
                if (m_VisualStyle == null)
                {
                    m_VisualStyle = new Syncfusion.Windows.Tools.Controls.RoutedCommand(VisualStyleExecute, VisualStyleCanExecute);
                }
                return m_VisualStyle;
            }
        }
        internal bool VisualStyleCanExecute(object parameter)
        {
            return true;
        }

        internal void VisualStyleExecute(object parameter)
        {
            string str = parameter.ToString();
            DependencyObject root = FindLogicalRoot(this);
            if (root != null)
            {
                SkinStorage.SetVisualStyle(root, str);
            }
        }
        private DependencyObject FindLogicalRoot(DependencyObject obj)
        {
            if (obj == null)
                return null;
            else
            {
                var parent = LogicalTreeHelper.GetParent(obj);
                return parent != null ? FindLogicalRoot(parent) : obj;
            }
        }

        public MainWindow()
        {
            InitializeComponent();

            rte.FileOpening += (sender, e) =>
                {
                    rte.Document = DocxImporting.ConvertToDocumentAdv(e.DocumentStream, e.FormatType);
                    PageCount = rte.Viewer.Pages.Count;
                };

            rte.FileSaving += (sender, e) =>
                {
                    DocxExporting.ConvertToDocument(rte.Document, e.DoucmentStream, e.FormatType);
                };

            rte.Loaded += (sender, e) =>
                {
                    PageCount = rte.Viewer.Pages.Count;
                };

            this.DataContextChanged += (sender, e) =>
            {
                RichTextBoxAdv richtext = e.NewValue as RichTextBoxAdv;
                if (richtext != null)
                {
                    rte = richtext;
                    if (m_VisualStyle != null)
                        m_VisualStyle.UpdateCanExecute();
                }
            };

            DataContext = rte;
            SkinStorage.SetVisualStyle(this, "Office2010Blue");

            InitCustomRibbonHelper();
        }

        private void InitCustomRibbonHelper()
        {
            // Customizing RichTextRibbon
            ribbon.Loaded += new RoutedEventHandler(OnLoaded);
            AddFontFamily();

            if (PART_FontFamilyCombo != null)
            {
                PART_FontFamilyCombo.DropDownClosed += (sender, e) =>
                {
                    if (rte != null)
                    {
                        RichTextBoxAdv.ChangeFontFamily.Execute(PART_FontFamilyCombo.SelectedItem, null);
                    }
                };
            }
            if (PART_FontSizeCombo != null)
            {
                PART_FontSizeCombo.DropDownClosed += (sender, e) =>
                {
                    if (rte != null)
                    {
                        RichTextBoxAdv.ChangeFontSize.Execute(PART_FontSizeCombo.SelectedItem, null);
                    }
                };
            }
            if (PART_Colorpicker != null)
            {
                PART_Colorpicker.ColorChanged += (sender, args) =>
                {
                    if (rte != null)
                    {
                        rte.Selection.ChangeForeground(PART_Colorpicker.Color);
                        SplitButton splitbutton = (SplitButton)PART_Colorpicker.Parent;
                        splitbutton.IsDropDownOpen = false;
                    }
                };
            }
            if (PART_GrowFont != null)
            {
                PART_GrowFont.Click += (sender, e) =>
                {
                    if (rte != null)
                    {
                        if (PART_FontSizeCombo.SelectedIndex < PART_FontSizeCombo.Items.Count - 1)
                        {
                            PART_FontSizeCombo.SelectedIndex += 1;
                            rte.Selection.ChangeFontSize(int.Parse((PART_FontSizeCombo.SelectedItem).ToString()));
                        }
                        else
                        {
                            rte.Selection.ChangeFontSize(rte.CurrentInlineStyle.FontSize + 10);
                        }
                    }
                };
            }
            if (PART_ShrinkFont != null)
                PART_ShrinkFont.Click += (sender, args) =>
                {
                    if (rte != null)
                    {
                        if (PART_FontSizeCombo.SelectedIndex > 0)
                        {
                            PART_FontSizeCombo.SelectedIndex -= 1;
                            rte.Selection.ChangeFontSize(int.Parse((PART_FontSizeCombo.SelectedItem).ToString()));
                        }
                        else
                        {
                            if (rte.CurrentInlineStyle.FontSize > 1)
                                rte.Selection.ChangeFontSize(rte.CurrentInlineStyle.FontSize - 1);
                        }
                    }
                };
            if (PART_TablePicker != null)
            {
                PART_TablePicker.Click += (sender, args) =>
                {
                    if (rte != null)
                    {
                        rte.InsertTableInBlocks(PART_TablePicker.SelectedCell.Row + 1, PART_TablePicker.SelectedCell.Column + 1);

                        object obj = PART_TablePicker.Parent;

                        while (!(obj is DropDownButton))
                        {
                            obj = (obj as FrameworkElement).Parent;
                        }
                        DropDownButton dropdown = obj as DropDownButton;
                        dropdown.IsDropDownOpen = false;
                    }
                };
            }
            if (PART_PrintDocument != null)
            {
                PART_PrintDocument.Click += (sender, e) =>
                {
                    ribbon.BackStageButton.IsOpen = false;
                };
            }
            if (PART_Open != null)
            {
                PART_Open.Click += (sender, e) =>
                {
                    ribbon.BackStageButton.IsOpen = false;
                };
            }
            if (PART_InsertTable != null)
            {
                PART_InsertTable.Click += (sender, args) =>
                {
                    object obj = PART_InsertTable.Parent;

                    while (!(obj is DropDownButton))
                    {
                        obj = (obj as FrameworkElement).Parent;
                    }
                    DropDownButton dropdown = obj as DropDownButton;
                    dropdown.IsDropDownOpen = false;
                };
            }
            if (PART_NewDocument_BackStage != null)
            {
                PART_NewDocument_BackStage.Click += (sender, e) =>
                {
                    if (ribbon != null)
                    {
                        ribbon.BackStageButton.IsOpen = false;
                        rte.DocumentTitle = "Untitled";
                    }
                };
            }

            if (PART_CreateDocument != null)
            {
                PART_CreateDocument.Click += (sender, e) =>
                {
                    if (ribbon != null)
                    {
                        ribbon.BackStageButton.IsOpen = false;
                        rte.DocumentTitle = "Untitled";
                    }

                };
            }

            if (PART_FontColorSplitButton != null)
                PART_FontColorSplitButton.Click += (sender, e) =>
                {
                    if (PART_Colorpicker != null && rte != null)
                    {
                        rte.Selection.ChangeForeground(PART_Colorpicker.Color);
                    }
                };
            if (ribbon != null)
            {
                ribbon.BackStageClosed += (sender, e) =>
                {
                    rte.Focus();
                };
            }
        }
        public FontFamilies FontFamilySource
        {
            get { return (FontFamilies)GetValue(FontFamilySourceProperty); }
            set { SetValue(FontFamilySourceProperty, value); }
        }

        // Using a DependencyProperty as the backing store for FontFamilySource.
        public static readonly DependencyProperty FontFamilySourceProperty =
            DependencyProperty.Register("FontFamilySource", typeof(FontFamilies), typeof(MainWindow), new PropertyMetadata(new FontFamilies()));
        public static readonly DependencyProperty FontSizeSourceProperty = DependencyProperty.Register("FontSizeSource", typeof(double[]), typeof(MainWindow),
            new PropertyMetadata(new double[] { 8, 9, 10, 11, 12, 14, 16, 18, 20, 24, 26, 28, 36, 48, 72 }));

        public double[] FontSizeSource
        {
            get
            {
                return (double[])GetValue(FontSizeSourceProperty);
            }
            set
            {
                SetValue(FontSizeSourceProperty, value);
            }
        }
        public int PageCount
        {
            get { return (int)GetValue(PageCountProperty); }
            set { SetValue(PageCountProperty, value); }
        }

        // Using a DependencyProperty as the backing store for PageCount.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty PageCountProperty =
            DependencyProperty.Register("PageCount", typeof(int), typeof(MainWindow), new PropertyMetadata(0));


        private void OnLoaded(object sender, RoutedEventArgs e)
        {
            Window window = this as Window;
            if (window != null)
            {
                window.CommandBindings.Add(new CommandBinding(RichTextBoxAdv.Save, SaveExecute));
                window.CommandBindings.Add(new CommandBinding(RichTextBoxAdv.SaveAs, SaveAsExecute));
                window.CommandBindings.Add(new CommandBinding(RichTextBoxAdv.Print, PrintExecute));
                window.CommandBindings.Add(new CommandBinding(RichTextBoxAdv.Open, OpenExecute));
                window.CommandBindings.Add(new CommandBinding(RichTextBoxAdv.New, NewExecute));
            }
            if (rte != null)
                rte.Focus();
        }
        private void SaveExecute(object sender, ExecutedRoutedEventArgs e)
        {
            if (rte != null)
            {
                rte.SaveDocument(string.Empty);
                rte.Focus();
            }
        }

        private void SaveAsExecute(object sender, ExecutedRoutedEventArgs e)
        {
            string extension = string.Empty;
            if (e.Parameter == null)
                extension = ".docx";
            else
                extension = e.Parameter.ToString();
            if (rte != null)
            {
                rte.SaveDocument(extension);
            }
        }

        private void PrintExecute(object sender, ExecutedRoutedEventArgs e)
        {
            if (rte != null)
            {
                rte.PrintDocument();
                rte.Focus();
            }
        }

        private void NewExecute(object sender, ExecutedRoutedEventArgs e)
        {
            if (rte != null)
            {
                rte.CreateEmptyDocument();
                rte.Focus();
            }
        }

        private void OpenExecute(object sender, ExecutedRoutedEventArgs e)
        {
            if (rte != null)
            {
                rte.OpenDocument();
                rte.Focus();
            }
        }

        private void AddFontFamily()
        {
            FontFamilySource = new FontFamilies();
            foreach (FontFamily fontfamily in Fonts.SystemFontFamilies)
            {
                FontFamilySource.Add(fontfamily);
            }
        }

    }
    public class FontFamilies : List<FontFamily>
    {
        public FontFamilies()
        {
            foreach (FontFamily fontfamily in Fonts.SystemFontFamilies)
            {
                Add(fontfamily);
            }
        }
    }
    public class ZoomFactorInPercentage : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            double factor = Math.Round(System.Convert.ToDouble(value), 2);
            if (factor == 0)
            {
                factor = 0.1;
            }
            return (int)(factor * 100 * 4);
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    public class RunAdv : Run
    {
        public RunAdv()
        {

        }

        public string RunText
        {
            get { return (string)GetValue(RunTextProperty); }
            set { SetValue(RunTextProperty, value); }
        }

        // Using a DependencyProperty as the backing store for RunText.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty RunTextProperty =
            DependencyProperty.Register("RunText", typeof(string), typeof(RunAdv), new PropertyMetadata(string.Empty, new PropertyChangedCallback(OnRunTextChanged)));

        private static void OnRunTextChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            RunAdv runadv = d as RunAdv;
            runadv.OnRunTextChanged(e);
        }

        private void OnRunTextChanged(DependencyPropertyChangedEventArgs e)
        {
            base.Text = (string)e.NewValue;
        }
    }
}
